@@ -12,17 +12,19 @@ import (
 	"time"
 
 	"github.com/google/uuid"
-	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/bsontype"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/x/bsonx/bsoncore"
 
-	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
-	plog "github.com/percona/percona-backup-mongodb/pbm/log"
+	"github.com/percona/percona-backup-mongodb/pbm/ctrl"
+	"github.com/percona/percona-backup-mongodb/pbm/defs"
+	"github.com/percona/percona-backup-mongodb/pbm/errors"
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/pbm/topo"
 )
 
 const cursorCreateRetries = 10
@@ -43,22 +45,22 @@ type BCoplogTS struct {
 // see https://www.percona.com/blog/2021/06/07/experimental-feature-backupcursorextend-in-percona-server-for-mongodb/
 type BackupCursorData struct {
 	Meta *Meta
-	Data []pbm.File
+	Data []File
 }
 
 type BackupCursor struct {
 	id    UUID
-	n     *pbm.Node
-	l     *plog.Event
+	conn  *mongo.Client
+	l     log.LogEvent
 	opts  bson.D
 	close chan struct{}
 
 	CustomThisID string
 }
 
-func NewBackupCursor(n *pbm.Node, l *plog.Event, opts bson.D) *BackupCursor {
+func NewBackupCursor(conn *mongo.Client, l log.LogEvent, opts bson.D) *BackupCursor {
 	return &BackupCursor{
-		n:    n,
+		conn: conn,
 		l:    l,
 		opts: opts,
 	}
@@ -84,7 +86,7 @@ func (bc *BackupCursor) create(ctx context.Context, retry int) (*mongo.Cursor, e
 			}
 		}
 
-		cur, err := bc.n.Session().Database("admin").Aggregate(ctx, mongo.Pipeline{
+		cur, err := bc.conn.Database("admin").Aggregate(ctx, mongo.Pipeline{
 			{{"$backupCursor", opts}},
 		})
 		if err != nil {
@@ -118,12 +120,12 @@ func (bc *BackupCursor) Data(ctx context.Context) (_ *BackupCursorData, err erro
 	}
 	defer func() {
 		if err != nil {
-			cur.Close(ctx)
+			cur.Close(context.Background())
 		}
 	}()
 
 	var m *Meta
-	var files []pbm.File
+	var files []File
 	for cur.TryNext(ctx) {
 		// metadata is the first
 		if m == nil {
@@ -138,7 +140,7 @@ func (bc *BackupCursor) Data(ctx context.Context) (_ *BackupCursorData, err erro
 			continue
 		}
 
-		var d pbm.File
+		var d File
 		err = cur.Decode(&d)
 		if err != nil {
 			return nil, errors.Wrap(err, "decode filename")
@@ -169,9 +171,9 @@ func (bc *BackupCursor) Data(ctx context.Context) (_ *BackupCursorData, err erro
 	return &BackupCursorData{m, files}, nil
 }
 
-func (bc *BackupCursor) Journals(upto primitive.Timestamp) ([]pbm.File, error) {
+func (bc *BackupCursor) Journals(upto primitive.Timestamp) ([]File, error) {
 	ctx := context.Background()
-	cur, err := bc.n.Session().Database("admin").Aggregate(ctx,
+	cur, err := bc.conn.Database("admin").Aggregate(ctx,
 		mongo.Pipeline{
 			{{"$backupCursorExtend", bson.D{{"backupId", bc.id}, {"timestamp", upto}}}},
 		})
@@ -180,7 +182,7 @@ func (bc *BackupCursor) Journals(upto primitive.Timestamp) ([]pbm.File, error) {
 	}
 	defer cur.Close(ctx)
 
-	var j []pbm.File
+	var j []File
 
 	err = cur.All(ctx, &j)
 	return j, err
@@ -192,24 +194,28 @@ func (bc *BackupCursor) Close() {
 	}
 }
 
+func backupCursorName(s string) string {
+	return strings.NewReplacer("-", "", ":", "").Replace(s)
+}
+
 func (b *Backup) doPhysical(
 	ctx context.Context,
-	bcp *pbm.BackupCmd,
-	opid pbm.OPID,
-	rsMeta *pbm.BackupReplset,
-	inf *pbm.NodeInfo,
+	bcp *ctrl.BackupCmd,
+	opid ctrl.OPID,
+	rsMeta *BackupReplset,
+	inf *topo.NodeInfo,
 	stg storage.Storage,
-	l *plog.Event,
+	l log.LogEvent,
 ) error {
 	currOpts := bson.D{}
-	if b.typ == pbm.IncrementalBackup {
+	if b.typ == defs.IncrementalBackup {
 		currOpts = bson.D{
 			// thisBackupName can be customized on retry
-			{"thisBackupName", pbm.BackupCursorName(bcp.Name)},
+			{"thisBackupName", backupCursorName(bcp.Name)},
 			{"incrementalBackup", true},
 		}
 		if !b.incrBase {
-			src, err := b.cn.LastIncrementalBackup()
+			src, err := LastIncrementalBackup(ctx, b.leadConn)
 			if err != nil {
 				return errors.Wrap(err, "define source backup")
 			}
@@ -219,7 +225,7 @@ func (b *Backup) doPhysical(
 
 			// ? should be done during Init()?
 			if inf.IsLeader() {
-				err := b.cn.SetSrcBackup(bcp.Name, src.Name)
+				err := SetSrcBackup(ctx, b.leadConn, bcp.Name, src.Name)
 				if err != nil {
 					return errors.Wrap(err, "set source backup in meta")
 				}
@@ -235,7 +241,7 @@ func (b *Backup) doPhysical(
 			}
 			if realSrcID == "" {
 				// no custom thisBackupName was used. fallback to default
-				realSrcID = pbm.BackupCursorName(src.Name)
+				realSrcID = backupCursorName(src.Name)
 			}
 
 			currOpts = append(currOpts, bson.E{"srcBackupName", realSrcID})
@@ -243,7 +249,7 @@ func (b *Backup) doPhysical(
 			// We don't need any previous incremental backup history if
 			// this is a base backup. So we can flush it to free up resources.
 			l.Debug("flush incremental backup history")
-			cr, err := NewBackupCursor(b.node, l, bson.D{
+			cr, err := NewBackupCursor(b.nodeConn, l, bson.D{
 				{"disableIncrementalBackup", true},
 			}).create(ctx, cursorCreateRetries)
 			if err != nil {
@@ -256,12 +262,12 @@ func (b *Backup) doPhysical(
 			}
 		}
 	}
-	cursor := NewBackupCursor(b.node, l, currOpts)
+	cursor := NewBackupCursor(b.nodeConn, l, currOpts)
 	defer cursor.Close()
 
 	bcur, err := cursor.Data(ctx)
 	if err != nil {
-		if b.typ == pbm.IncrementalBackup && strings.Contains(err.Error(), "(UnknownError) 2: No such file or directory") {
+		if b.typ == defs.IncrementalBackup && strings.Contains(err.Error(), "(UnknownError) 2: No such file or directory") {
 			return errors.New("can't find incremental backup history." +
 				" Previous backup was made on another node." +
 				" You can make a new base incremental backup to start a new history.")
@@ -271,36 +277,36 @@ func (b *Backup) doPhysical(
 
 	l.Debug("backup cursor id: %s", bcur.Meta.ID)
 
-	lwts, err := pbm.LastWrite(b.node.Session(), true)
+	lwts, err := topo.GetLastWrite(ctx, b.nodeConn, true)
 	if err != nil {
 		return errors.Wrap(err, "get shard's last write ts")
 	}
 
-	defOpts := &pbm.MongodOpts{}
+	defOpts := &topo.MongodOpts{}
 	defOpts.Storage.WiredTiger.EngineConfig.JournalCompressor = "snappy"
 	defOpts.Storage.WiredTiger.CollectionConfig.BlockCompressor = "snappy"
 	defOpts.Storage.WiredTiger.IndexConfig.PrefixCompression = true
 
-	mopts, err := b.node.GetOpts(defOpts)
+	mopts, err := topo.GetMongodOpts(ctx, b.nodeConn, defOpts)
 	if err != nil {
 		return errors.Wrap(err, "get mongod options")
 	}
 
 	rsMeta.MongodOpts = mopts
-	rsMeta.Status = pbm.StatusRunning
+	rsMeta.Status = defs.StatusRunning
 	rsMeta.FirstWriteTS = bcur.Meta.OplogEnd.TS
 	rsMeta.LastWriteTS = lwts
 	if cursor.CustomThisID != "" {
 		// custom thisBackupName was used
 		rsMeta.CustomThisID = cursor.CustomThisID
 	}
-	err = b.cn.AddRSMeta(bcp.Name, *rsMeta)
+	err = AddRSMeta(ctx, b.leadConn, bcp.Name, *rsMeta)
 	if err != nil {
 		return errors.Wrap(err, "add shard's metadata")
 	}
 
 	if inf.IsLeader() {
-		err := b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusRunning, ref(b.timeouts.StartingStatus()))
+		err := b.reconcileStatus(ctx, bcp.Name, opid.String(), defs.StatusRunning, ref(b.timeouts.StartingStatus()))
 		if err != nil {
 			if errors.Is(err, errConvergeTimeOut) {
 				return errors.Wrap(err, "couldn't get response from all shards")
@@ -308,24 +314,24 @@ func (b *Backup) doPhysical(
 			return errors.Wrap(err, "check cluster for backup started")
 		}
 
-		err = b.setClusterFirstWrite(bcp.Name)
+		err = b.setClusterFirstWrite(ctx, bcp.Name)
 		if err != nil {
 			return errors.Wrap(err, "set cluster first write ts")
 		}
 
-		err = b.setClusterLastWrite(bcp.Name)
+		err = b.setClusterLastWriteForPhysical(ctx, bcp.Name)
 		if err != nil {
 			return errors.Wrap(err, "set cluster last write ts")
 		}
 	}
 
 	// Waiting for cluster's StatusRunning to move further.
-	err = b.waitForStatus(bcp.Name, pbm.StatusRunning, nil)
+	err = b.waitForStatus(ctx, bcp.Name, defs.StatusRunning, nil)
 	if err != nil {
 		return errors.Wrap(err, "waiting for running")
 	}
 
-	_, lwTS, err := b.waitForFirstLastWrite(bcp.Name)
+	_, lwTS, err := b.waitForFirstLastWrite(ctx, bcp.Name)
 	if err != nil {
 		return errors.Wrap(err, "get cluster first & last write ts")
 	}
@@ -347,22 +353,23 @@ func (b *Backup) doPhysical(
 		data = append(data, *stgb)
 	}
 
-	if b.typ == pbm.ExternalBackup {
-		return b.handleExternal(bcp, rsMeta, data, jrnls, bcur.Meta.DBpath, opid, inf, l)
+	if b.typ == defs.ExternalBackup {
+		return b.handleExternal(ctx, bcp, rsMeta, data, jrnls, bcur.Meta.DBpath, opid, inf, l)
 	}
 
 	return b.uploadPhysical(ctx, bcp, rsMeta, data, jrnls, bcur.Meta.DBpath, stg, l)
 }
 
 func (b *Backup) handleExternal(
-	bcp *pbm.BackupCmd,
-	rsMeta *pbm.BackupReplset,
+	ctx context.Context,
+	bcp *ctrl.BackupCmd,
+	rsMeta *BackupReplset,
 	data,
-	jrnls []pbm.File,
+	jrnls []File,
 	dbpath string,
-	opid pbm.OPID,
-	inf *pbm.NodeInfo,
-	l *plog.Event,
+	opid ctrl.OPID,
+	inf *topo.NodeInfo,
+	l log.LogEvent,
 ) error {
 	for _, f := range append(data, jrnls...) {
 		f.Name = path.Clean("./" + strings.TrimPrefix(f.Name, dbpath))
@@ -374,15 +381,15 @@ func (b *Backup) handleExternal(
 	// original LastWriteTS in the meta stored on PBM storage. As rsMeta might
 	// be used outside of this method.
 	fsMeta := *rsMeta
-	bmeta, err := b.cn.GetBackupMeta(bcp.Name)
+	bmeta, err := NewDBManager(b.leadConn).GetBackupByName(ctx, bcp.Name)
 	if err == nil {
 		fsMeta.LastWriteTS = bmeta.LastWriteTS
 	} else {
 		l.Warning("define LastWriteTS: get backup meta: %v", err)
 	}
 	// save rs meta along with the data files so it can be used during the restore
-	metaf := fmt.Sprintf(pbm.ExternalRsMetaFile, fsMeta.Name)
-	fsMeta.Files = append(fsMeta.Files, pbm.File{
+	metaf := fmt.Sprintf(defs.ExternalRsMetaFile, fsMeta.Name)
+	fsMeta.Files = append(fsMeta.Files, File{
 		Name: metaf,
 	})
 	metadst := filepath.Join(dbpath, metaf)
@@ -392,20 +399,20 @@ func (b *Backup) handleExternal(
 		l.Warning("failed to save rs meta file <%s>: %v", metadst, err)
 	}
 
-	err = b.cn.RSSetPhyFiles(bcp.Name, rsMeta.Name, rsMeta)
+	err = RSSetPhyFiles(ctx, b.leadConn, bcp.Name, rsMeta.Name, rsMeta)
 	if err != nil {
 		return errors.Wrap(err, "set shard's files list")
 	}
 
-	err = b.toState(pbm.StatusCopyReady, bcp.Name, opid.String(), inf, nil)
+	err = b.toState(ctx, defs.StatusCopyReady, bcp.Name, opid.String(), inf, nil)
 	if err != nil {
-		return errors.Wrapf(err, "converge to %s", pbm.StatusCopyReady)
+		return errors.Wrapf(err, "converge to %s", defs.StatusCopyReady)
 	}
 
 	l.Info("waiting for the datadir to be copied")
-	err = b.waitForStatus(bcp.Name, pbm.StatusCopyDone, nil)
+	err = b.waitForStatus(ctx, bcp.Name, defs.StatusCopyDone, nil)
 	if err != nil {
-		return errors.Wrapf(err, "waiting for %s", pbm.StatusCopyDone)
+		return errors.Wrapf(err, "waiting for %s", defs.StatusCopyDone)
 	}
 
 	err = os.Remove(metadst)
@@ -416,7 +423,7 @@ func (b *Backup) handleExternal(
 	return nil
 }
 
-func writeRSmetaToDisk(fname string, rsMeta *pbm.BackupReplset) error {
+func writeRSmetaToDisk(fname string, rsMeta *BackupReplset) error {
 	fw, err := os.OpenFile(fname, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0o600)
 	if err != nil {
 		return errors.Wrapf(err, "create/open")
@@ -439,18 +446,18 @@ func writeRSmetaToDisk(fname string, rsMeta *pbm.BackupReplset) error {
 
 func (b *Backup) uploadPhysical(
 	ctx context.Context,
-	bcp *pbm.BackupCmd,
-	rsMeta *pbm.BackupReplset,
+	bcp *ctrl.BackupCmd,
+	rsMeta *BackupReplset,
 	data,
-	jrnls []pbm.File,
+	jrnls []File,
 	dbpath string,
 	stg storage.Storage,
-	l *plog.Event,
+	l log.LogEvent,
 ) error {
 	var err error
 	l.Info("uploading data")
 	rsMeta.Files, err = uploadFiles(ctx, data, bcp.Name+"/"+rsMeta.Name, dbpath,
-		b.typ == pbm.IncrementalBackup, stg, bcp.Compression, bcp.CompressionLevel, l)
+		b.typ == defs.IncrementalBackup, stg, bcp.Compression, bcp.CompressionLevel, l)
 	if err != nil {
 		return err
 	}
@@ -465,7 +472,7 @@ func (b *Backup) uploadPhysical(
 	l.Info("uploading journals done")
 	rsMeta.Files = append(rsMeta.Files, ju...)
 
-	err = b.cn.RSSetPhyFiles(bcp.Name, rsMeta.Name, rsMeta)
+	err = RSSetPhyFiles(ctx, b.leadConn, bcp.Name, rsMeta.Name, rsMeta)
 	if err != nil {
 		return errors.Wrap(err, "set shard's files list")
 	}
@@ -475,7 +482,7 @@ func (b *Backup) uploadPhysical(
 		size += f.StgSize
 	}
 
-	err = b.cn.IncBackupSize(ctx, bcp.Name, size)
+	err = IncBackupSize(ctx, b.leadConn, bcp.Name, size)
 	if err != nil {
 		return errors.Wrap(err, "inc backup size")
 	}
@@ -485,7 +492,7 @@ func (b *Backup) uploadPhysical(
 
 const storagebson = "storage.bson"
 
-func getStorageBSON(dbpath string) (*pbm.File, error) {
+func getStorageBSON(dbpath string) (*File, error) {
 	f, err := os.Stat(path.Join(dbpath, storagebson))
 	if err != nil {
 		if errors.Is(err, os.ErrNotExist) {
@@ -494,7 +501,7 @@ func getStorageBSON(dbpath string) (*pbm.File, error) {
 		return nil, err
 	}
 
-	return &pbm.File{
+	return &File{
 		Name:  path.Join(dbpath, storagebson),
 		Len:   f.Size(),
 		Size:  f.Size(),
@@ -539,15 +546,15 @@ func (id *UUID) IsZero() bool {
 // what files shouldn't be restored (those which isn't in the target backup).
 func uploadFiles(
 	ctx context.Context,
-	files []pbm.File,
+	files []File,
 	subdir string,
 	trimPrefix string,
 	incr bool,
 	stg storage.Storage,
 	comprT compress.CompressionType,
 	comprL *int,
-	l *plog.Event,
-) ([]pbm.File, error) {
+	l log.LogEvent,
+) ([]File, error) {
 	if len(files) == 0 {
 		return nil, nil
 	}
@@ -559,11 +566,11 @@ func uploadFiles(
 	}
 
 	wfile := files[0]
-	data := []pbm.File{}
+	data := []File{}
 	for _, file := range files[1:] {
 		select {
 		case <-ctx.Done():
-			return nil, ErrCancelled
+			return nil, storage.ErrCancelled
 		default:
 		}
 
@@ -616,13 +623,13 @@ func uploadFiles(
 
 func writeFile(
 	ctx context.Context,
-	src pbm.File,
+	src File,
 	dst string,
 	stg storage.Storage,
 	compression compress.CompressionType,
 	compressLevel *int,
-	l *plog.Event,
-) (*pbm.File, error) {
+	l log.LogEvent,
+) (*File, error) {
 	fstat, err := os.Stat(src.Name)
 	if err != nil {
 		return nil, errors.Wrap(err, "get file stat")
@@ -641,7 +648,7 @@ func writeFile(
 	}
 	l.Debug("uploading: %s %s", src, fmtSize(sz))
 
-	_, err = Upload(ctx, &src, stg, compression, compressLevel, dst, sz)
+	_, err = storage.Upload(ctx, &src, stg, compression, compressLevel, dst, sz)
 	if err != nil {
 		return nil, errors.Wrap(err, "upload file")
 	}
@@ -651,7 +658,7 @@ func writeFile(
 		return nil, errors.Wrapf(err, "get storage file stat %s", dst)
 	}
 
-	return &pbm.File{
+	return &File{
 		Name:    src.Name,
 		Size:    fstat.Size(),
 		Fmode:   fstat.Mode(),
